{
  "type": "record",
  "name": "DataHubIngestionSourceInfo",
  "namespace": "com.linkedin.pegasus2avro.ingestion",
  "doc": "Info about a DataHub ingestion source",
  "fields": [
    {
      "name": "name",
      "type": "string",
      "doc": "The display name of the ingestion source",
      "Searchable": {
        "fieldType": "TEXT_PARTIAL"
      }
    },
    {
      "name": "type",
      "type": "string",
      "doc": "The type of the source itself, e.g. mysql, bigquery, bigquery-usage. Should match the recipe."
    },
    {
      "name": "platform",
      "type": [
        "null",
        "string"
      ],
      "doc": "Data Platform URN associated with the source",
      "default": null,
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
      }
    },
    {
      "name": "schedule",
      "type": [
        "null",
        {
          "type": "record",
          "name": "DataHubIngestionSourceSchedule",
          "doc": "The schedule associated with an ingestion source.",
          "fields": [
            {
              "name": "interval",
              "type": "string",
              "doc": "A cron-formatted execution interval, as a cron string, e.g. * * * * *"
            },
            {
              "name": "timezone",
              "type": "string",
              "doc": "Timezone in which the cron interval applies, e.g. America/Los Angeles"
            }
          ]
        }
      ],
      "doc": "The schedule on which the ingestion source is executed",
      "default": null
    },
    {
      "name": "config",
      "type": {
        "type": "record",
        "name": "DataHubIngestionSourceConfig",
        "fields": [
          {
            "name": "recipe",
            "type": "string",
            "doc": "The JSON recipe to use for ingestion"
          },
          {
            "name": "version",
            "type": [
              "null",
              "string"
            ],
            "doc": "The PyPI version of the datahub CLI to use when executing a recipe",
            "default": null
          },
          {
            "name": "executorId",
            "type": [
              "null",
              "string"
            ],
            "doc": "The id of the executor to use to execute the ingestion run",
            "default": null
          },
          {
            "name": "debugMode",
            "type": [
              "null",
              "boolean"
            ],
            "doc": "Whether or not to run this ingestion source in debug mode",
            "default": null
          }
        ]
      },
      "doc": "Parameters associated with the Ingestion Source"
    }
  ],
  "Aspect": {
    "name": "dataHubIngestionSourceInfo"
  }
}